CREATE TABLE RESOURCE_PROVIDERS (
  PROVIDER_ID VARCHAR(20) NOT NULL,
  RESOURCE_DN VARCHAR(2000),
  RECON_STATUS SMALLINT,
  LAST_RECON_TIME TIMESTAMP,
  MAX_RECON_DURATION BIGINT,
  LOCK_SERVICE SMALLINT,
  REQUEST_ID VARCHAR(20),
  CURRENT_REQUEST_COUNT		BIGINT,
  MAX_REQUEST_COUNT		BIGINT,
  LAST_RESPONSE_TIME		TIMESTAMP,
  RESOURCE_STATUS		SMALLINT,
  CONSTRAINT RES_PROV_PK PRIMARY KEY(PROVIDER_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE REMOTE_SERVICES_REQUESTS (
  PROVIDER_ID VARCHAR(20),
  REQUEST_ID VARCHAR(20) NOT NULL,
  TYPE SMALLINT,
  OPERATION SMALLINT,
  REQUEST_TIME TIMESTAMP,
  EXPIRATION_TIME TIMESTAMP,
  TARGET VARCHAR(2000),
  SERVICE_DN VARCHAR(2000) NOT NULL,
  DATA CLOB(10M),
  CONNECTION_POINT BLOB(10M),
  CONSTRAINT REMOTE_SVR_REQ_FK FOREIGN KEY (PROVIDER_ID) REFERENCES RESOURCE_PROVIDERS (PROVIDER_ID),
  CONSTRAINT REMOTE_SVR_REQ_PK PRIMARY KEY(REQUEST_ID)
)  IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE REMOTE_RESOURCES_RECONS (
  PROVIDER_ID VARCHAR(20) NOT NULL,
  RECON_ID BIGINT NOT NULL,
  DAY_OF_MONTH INTEGER,
  MONTH_NUM INTEGER,
  DAY_OF_WEEK INTEGER,
  HOUR_NUM INTEGER,
  MINUTE_NUM INTEGER,
  MAX_DURATION BIGINT,
  LOCK_SERVICE SMALLINT DEFAULT 1,
  CONSTRAINT REM_RES_RECONS_FK FOREIGN KEY(PROVIDER_ID) REFERENCES RESOURCE_PROVIDERS(PROVIDER_ID),
  CONSTRAINT REM_RES_RECONS_PK PRIMARY KEY(PROVIDER_ID, RECON_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE REMOTE_RESOURCES_RECON_QUERIES (
  PROVIDER_ID VARCHAR(20) NOT NULL,
  RECON_ID BIGINT NOT NULL,
  QUERY_ID BIGINT NOT NULL,
  RECON_FILTER VARCHAR(4000),
  RECON_BASE VARCHAR(4000),
  MAX_DURATION BIGINT,
  MAX_ENTRIES BIGINT,
  ATTRIBUTES CLOB(10M),
  CONSTRAINT REM_REC_QUERIES_FK FOREIGN KEY(PROVIDER_ID, RECON_ID) REFERENCES REMOTE_RESOURCES_RECONS(PROVIDER_ID, RECON_ID),
  CONSTRAINT REM_REC_QUERIES_PK PRIMARY KEY(PROVIDER_ID, RECON_ID, QUERY_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE SCHEDULED_MESSAGE (
  SCHEDULED_TIME BIGINT,
  SCHEDULED_MESSAGE_ID BIGINT NOT NULL,
  MESSAGE CLOB(10M),
  SERVER VARCHAR(20),
  CHECKPOINT_TIME BIGINT,
  REFERENCE_ID BIGINT,
  CONSTRAINT SCHED_MSG_PK PRIMARY KEY( SCHEDULED_MESSAGE_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE INDEX SCH_MSG_TIME_X ON SCHEDULED_MESSAGE (SCHEDULED_TIME, SERVER);

CREATE INDEX SCH_MSG_CHKPNT_X ON SCHEDULED_MESSAGE (CHECKPOINT_TIME);

CREATE INDEX SCH_MSG_REF_X ON SCHEDULED_MESSAGE (REFERENCE_ID);

CREATE TABLE NEXTVALUE (
  ID VARCHAR(50) NOT NULL,
  NEXTID INTEGER NOT NULL,
  CONSTRAINT NEXTVALUE_PK PRIMARY KEY(ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE PROCESS (
  ID BIGINT NOT NULL,
  PARENT_ID BIGINT,
  PARENT_ACTIVITY_ID BIGINT,
  NAME VARCHAR(100),
  TYPE VARCHAR(2) NOT NULL,
  DEFINITION_ID VARCHAR(2000) NOT NULL,
  REQUESTER_TYPE VARCHAR(2),
  REQUESTER VARCHAR(2000),
  REQUESTER_NAME VARCHAR(100),
  DESCRIPTION VARCHAR(300),
  PRIORITY INTEGER,
  SCHEDULED VARCHAR(50),
  STARTED VARCHAR(50),
  COMPLETED VARCHAR(50),
  LASTMODIFIED VARCHAR(50),
  SUBMITTED VARCHAR(50),
  STATE VARCHAR(1),
  NOTIFY INTEGER,
  REQUESTEE VARCHAR(2000),
  REQUESTEE_NAME VARCHAR(100),
  SUBJECT VARCHAR(2000),
  COMMENTS VARCHAR(200),
  RESULT_SUMMARY VARCHAR(2),
  RESULT_DETAIL CLOB(10M),
  TENANT VARCHAR(2000),
  CONSTRAINT PROCESS_PK PRIMARY KEY(ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE INDEX PROCESS_STATE_X ON PROCESS (STATE ASC);
CREATE INDEX PROCESS_PID_X ON PROCESS (PARENT_ID ASC);
CREATE INDEX PROCESS_VPC1_X ON PROCESS (PARENT_ID ASC, SUBMITTED DESC, ID DESC, STATE ASC) ALLOW REVERSE SCANS;

CREATE TABLE ACTIVITY (
  ID BIGINT NOT NULL,
  PROCESS_ID BIGINT NOT NULL,
  DEFINITION_ID VARCHAR(100) NOT NULL,
  ACTIVITY_INDEX INTEGER,
  LOOP_COUNT INTEGER,
  LOOP_RUNCOUNT INTEGER,
  RETRY_COUNT INTEGER,
  LOCK_COUNT INTEGER,
  SUBPROCESS_ID BIGINT,
  NAME VARCHAR(100),
  DESCRIPTION VARCHAR(300),
  TYPE VARCHAR(1),
  SUBTYPE VARCHAR(2),
  PRIORITY INTEGER,
  STARTED VARCHAR(50),
  COMPLETED VARCHAR(50),
  LASTMODIFIED VARCHAR(50),
  STATE VARCHAR(1),
  RESULT_SUMMARY VARCHAR(2),
  RESULT_DETAIL CLOB(10M),
  CONSTRAINT ACTIVITY_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
  CONSTRAINT ACTIVITY_PK PRIMARY KEY(ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE UNIQUE INDEX ACTIVITY_KEY ON ACTIVITY (PROCESS_ID, DEFINITION_ID, ACTIVITY_INDEX);
CREATE UNIQUE INDEX ACTIVITY_ID_X ON ACTIVITY (ID ASC) INCLUDE (SUBTYPE);
CREATE INDEX ACTIVITY_PID_X ON ACTIVITY (PROCESS_ID DESC);
CREATE INDEX ACTIVITY_STATE_X ON ACTIVITY (STATE DESC);


CREATE TABLE PROCESSDATA (
  PROCESS_ID BIGINT NOT NULL,
  DEF_ID VARCHAR(100) NOT NULL,
  NAME VARCHAR(100),
  CONTEXT VARCHAR(100),
  DESCRIPTION VARCHAR(300),
  TYPE VARCHAR(500),
  COLLECTION_TYPE VARCHAR(500),
  VALUE CLOB(10M),
  SMALL_VALUE VARCHAR(4000),
  CONSTRAINT PROCESSDATA_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
  CONSTRAINT PROCESSDATA_PK PRIMARY KEY(PROCESS_ID,DEF_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE INDEX PROCESSDATA_PID_X ON PROCESSDATA (PROCESS_ID DESC, DEF_ID DESC);

CREATE TABLE PROCESSLOG (
  ID BIGINT NOT NULL,
  PROCESS_ID BIGINT,
  ACTIVITY_ID BIGINT,
  CREATED VARCHAR(50),
  EVENTTYPE VARCHAR(2) NOT NULL,
  OLD_PARTICIPANT_TYPE VARCHAR(2),
  OLD_PARTICIPANT_ID VARCHAR(2000),
  NEW_PARTICIPANT_TYPE VARCHAR(2),
  NEW_PARTICIPANT_ID VARCHAR(2000),
  REQUESTOR_TYPE VARCHAR(2),
  REQUESTOR VARCHAR(2000),
  OLD_STATE VARCHAR(1),
  NEW_STATE VARCHAR(1),
  DATA_ID VARCHAR(100),
  NEW_DATA CLOB(10M),
  CONSTRAINT PROCESSLOG_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
  CONSTRAINT PROCESSLOG_PK PRIMARY KEY(ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE WORKITEM (
  ID BIGINT NOT NULL,
  PROCESS_ID BIGINT NOT NULL,
  ACTIVITY_ID BIGINT NOT NULL,
  PARTICIPANT_TYPE VARCHAR(2),
  PARTICIPANT VARCHAR(2000),
  CREATED VARCHAR(50),
  INPUT_PARAMETERS CLOB(10M),
  CONSTRAINT WORKITEM2_FK FOREIGN KEY(ACTIVITY_ID)
	REFERENCES ACTIVITY(ID),
  CONSTRAINT WORKITEM_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
  CONSTRAINT WORKITEM_PK PRIMARY KEY(ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE INDEX WORKITEM_PID_X ON WORKITEM (PROCESS_ID DESC);

CREATE TABLE PASSWORD_TRANSACTION (
  TRANSACTION_ID BIGINT NOT NULL,
  ACCOUNT_DN VARCHAR(2000),
  CREATION_DATE VARCHAR(50),
  PROCESS_ID BIGINT NOT NULL,
  ACTIVITY_ID BIGINT NOT NULL,
  PASSWORD VARCHAR(500),
  CONSTRAINT PWD_TRANS_PK PRIMARY KEY(TRANSACTION_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE PENDING (
  PROCESS_ID BIGINT NOT NULL,
  PERSON_DN VARCHAR(255),
  SERVICE_DN VARCHAR(2000),
  CONSTRAINT PENDING_FK FOREIGN KEY(PROCESS_ID)
	REFERENCES PROCESS(ID),
  CONSTRAINT PENDING_PK PRIMARY KEY(PROCESS_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE INDEX PENDING_FOR_PERSON ON PENDING (PERSON_DN);


CREATE TABLE LISTDATA
(
	DATA_ID	 BIGINT NOT NULL,
	INDEX_ID	INT NOT NULL,
	VALUE	   CLOB(10M),
CONSTRAINT LISTDATA_PK PRIMARY KEY(DATA_ID, INDEX_ID)
) IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE REPORT (
	ID INT NOT NULL,
	TITLE VARCHAR(255) NOT NULL,
	TYPE VARCHAR(255),
	AUTHOR VARCHAR(255),
	REPORT_SIZE INT,
	REPORT_DATA BLOB(10M),
	STYLESHEET_NAME VARCHAR(255),

	CONSTRAINT REPORT_P PRIMARY KEY (ID)
)IN ENROLE_DATA;

CREATE TABLE ENTITY_COLUMN (
	ENTITY_NAME VARCHAR(255) NOT NULL,
	COLUMN_NAME VARCHAR(255) NOT NULL,
	ATTRIBUTE_NAME VARCHAR(255) NOT NULL,
	MULTI_VALUED CHAR(1),
	AVAILABLE_FOR_REPORTING VARCHAR (255),
	TABLE_NAME VARCHAR (255),

	CONSTRAINT ENTITY_COLUMNP PRIMARY KEY (ENTITY_NAME, ATTRIBUTE_NAME)
)IN ENROLE_DATA;


CREATE TABLE COLUMN_REPORT (
	ENTITY_NAME VARCHAR(255) NOT NULL,
	COLUMN_NAME VARCHAR(255) NOT NULL,
	REPORT_ID INT NOT NULL,

	CONSTRAINT COLUMN_REPORT_P PRIMARY KEY (REPORT_ID, ENTITY_NAME, COLUMN_NAME)
)IN ENROLE_DATA;

CREATE TABLE ACI (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	SCOPE VARCHAR(255),
	TARGET VARCHAR(255) NOT NULL,
	PARENT VARCHAR(255),
	PRIMARY KEY(DN, NAME, TARGET)
)IN ENROLE_DATA;

CREATE TABLE ACI_ROLEDNS (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	ROLEDN VARCHAR(255) NOT NULL
)IN ENROLE_DATA;


CREATE TABLE ACI_PRINCIPALS (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	PRINCIPALNAME VARCHAR(255) NOT NULL
)IN ENROLE_DATA;

CREATE TABLE ACI_PERMISSION_ATTRIBUTERIGHT (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	ACTION VARCHAR(6) NOT NULL,
	OPERATION VARCHAR(3) NOT NULL,
	ATTRIBUTERIGHT VARCHAR(255) NOT NULL
)IN ENROLE_DATA;


CREATE TABLE ACI_PERMISSION_CLASSRIGHT (
	DN VARCHAR(255) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	TARGET VARCHAR(255) NOT NULL,
	ACTION VARCHAR(6) NOT NULL,
	CLASSRIGHT VARCHAR(255) NOT NULL
)IN ENROLE_DATA;


CREATE TABLE ENTITLEMENT (
	DN VARCHAR(255) NOT NULL,
	TYPE VARCHAR(255) NOT NULL,
	SERVICETARGETTYPE VARCHAR(255),
	SERVICETARGETNAME VARCHAR(255),
	PROCESSDN VARCHAR(255)
)IN ENROLE_DATA;

CREATE TABLE ENTITLEMENT_PROVISIONINGPARAMS (
	DN VARCHAR(255) NOT NULL,
	ATTRIBUTEVALUE VARCHAR(4000) NOT NULL,
	NAME VARCHAR(255) NOT NULL,
	ENFORCEMENT VARCHAR(255),
	EXPRTYPE VARCHAR(255)
)IN ENROLE_DATA;

CREATE TABLE SYNCHRONIZATION_HISTORY (
	SYNC_ID BIGINT  NOT NULL ,
	REQUESTOR VARCHAR (255) NOT NULL,
	REQ_TYPE VARCHAR (255),
	REQ_NAME VARCHAR (255),
	STATUS VARCHAR (255),
	TENANT VARCHAR (255),
	STATUS_DETAIL VARCHAR (255),
	SCHEDULED_TIME BIGINT,
	SUBMITTED_TIME BIGINT,
	STARTED_TIME BIGINT NOT NULL,
	COMPLETED_TIME BIGINT,
	SERVER_NAME VARCHAR (255),
	PRIMARY KEY (SYNC_ID, STARTED_TIME)
)IN ENROLE_DATA;


CREATE TABLE SYNCHRONIZATION_LOCK (
	HOST VARCHAR (255) NOT NULL,
	PRIMARY KEY (HOST)
)IN ENROLE_DATA;


CREATE TABLE RESOURCES_SYNCHRONIZATIONS (
	SYNC_ID BIGINT  NOT NULL,
	DAY_OF_MONTH INTEGER  NOT NULL,
	MONTH_NUM INTEGER  NOT NULL,
	DAY_OF_WEEK INTEGER  NOT NULL,
	HOUR_NUM INTEGER  NOT NULL,
	MINUTE_NUM INTEGER  NOT NULL,
	MAX_DURATION BIGINT,
	PRIMARY KEY (SYNC_ID)
)IN ENROLE_DATA;


CREATE TABLE CHANGELOG (
	CHANGE_NUMBER INTEGER NOT NULL
)IN ENROLE_DATA;

CREATE TABLE AUTH_KEY (
  Y VARCHAR(2000) NOT NULL,
  P VARCHAR(2000) NOT NULL,
  Q VARCHAR(2000) NOT NULL,
  G VARCHAR(2000) NOT NULL,
  X VARCHAR(2000) NOT NULL
)IN ENROLE_DATA;

CREATE TABLE RECONCILIATION (
  RECONID VARCHAR(255) NOT NULL,
  SERVICEDN VARCHAR(2000) NOT NULL,
  PROCESSEDACCOUNTS INTEGER NOT NULL,
  LOCALACCOUNTS INTEGER NOT NULL,
  TIMUSERACCOUNTS INTEGER NOT NULL,
  POLICYVIOLATIONS INTEGER NOT NULL,
  STARTED TIMESTAMP NOT NULL,
  COMPLETED TIMESTAMP NOT NULL,
  PRIMARY KEY (RECONID)
)IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

CREATE TABLE RECONCILIATION_INFO (
  RECONID VARCHAR(255) NOT NULL,
  ACCOUNTID VARCHAR(255),
  POLICYCOMPLIANCESTATUS VARCHAR(20),
  USERNAME VARCHAR(255),
  OPERATION VARCHAR(20),
  REMARKS VARCHAR(1000),
  FOREIGN KEY (RECONID) REFERENCES RECONCILIATION (RECONID)
)IN ENROLE_DATA INDEX IN ENROLE_INDEXES;

COMMIT;

